#-----------------------------------------------------------------------------
#
#   ALHM - Dialogs
#
#-----------------------------------------------------------------------------

from pygame import Rect
from pygame.locals import *
from albow.widget import Widget
from albow.layout import Row, Column
from albow.dialogs import Modal
from albow.resource import get_image
from albow.image_array import get_image_array
from albow.controls import ImageButton

class DualImageButton(ImageButton):

    def __init__(self, fg_image, bg_images, **kwds):
        if isinstance(fg_image, basestring):
            fg_image = get_image(fg_image)
        if isinstance(bg_images, basestring):
            bg_images = get_image_array(bg_images, 2)
        ImageButton.__init__(self, fg_image, **kwds)
        self.bg_image = bg_images[0]
        self.size = bg_images[0].get_size()
        self.highlighted_bg_image = bg_images[1]

class ImageDialog(Modal, Widget):

    click_outside_response = None
    margin = 20

    def __init__(self, bg_image, button_bg_images, client = None, responses = None,
            default = 0, cancel = -1, **kwds):
        if isinstance(bg_image, basestring):
            bg_image = get_image(bg_image)
        Widget.__init__(self, rect = bg_image.get_rect(), bg_image = bg_image, **kwds)
        if client or responses:
            rows = []
            w1 = 0
            w2 = 0
            if client:
                rows.append(client)
                w1 = client.width
            if responses:
                buttons = Row([
                    DualImageButton(image, button_bg_images, action = lambda r=response: self.dismiss(r))
                        for (response, image) in responses])
                rows.append(buttons)
                w2 = buttons.width
            if w1 < w2:
                a = 'l'
            else:
                a = 'r'
            contents = Column(rows, align = a)
            m = self.margin
            contents.bottomright = (self.width - m, self.height - m)
            self.add(contents)
            #self.shrink_wrap()
        if responses and default is not None:
            self.enter_response = responses[default]
        if responses and cancel is not None:
            self.cancel_response = responses[cancel]
    
    def mouse_down(self, e):
        if not e in self:
            response = self.click_outside_response
            if response is not None:
                self.dismiss(response)
